﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Life
{
    class Plansza
    {
        private int[,] plansza = null;

        private int szerokosc;
        public int Szerokosc 
        {
            get { return szerokosc; }
        }

        private int wysokosc;
        public int Wysokosc 
        {
            get { return wysokosc; }
        }

        private char znak;
        public char Znak 
        { 
            get { return znak; }
            set {znak = value; }
        }

        private ConsoleColor kolorTla = ConsoleColor.Green;
        private ConsoleColor kolorZnakow = ConsoleColor.Red;
        private int liczbaZywych = 0;
        private int numerCyklu = 0;

        public Plansza():this(10, 10, 0.24)
        { 
        }

        public Plansza(int wysokosc, int szerokosc, double prawdopodobienstwo)
        {
            this.wysokosc = wysokosc;
            this.szerokosc = szerokosc;
            plansza = new int[szerokosc, wysokosc];
            znak = '*';

            Inicjuj(prawdopodobienstwo);
        }

        public void Inicjuj(double prawdopodobienstwo)
        {
            System.Random rnd = new Random();
            liczbaZywych = 0;
            for(int i=0;i<szerokosc;i++)
                for (int j = 0; j < wysokosc; j++)
                {
                    if (rnd.NextDouble() < prawdopodobienstwo)
                    {
                        plansza[i, j] = 1;
                        liczbaZywych++;
                    }
                    else plansza[i, j] = 0;
                }
            numerCyklu = 0;
        }

        public void Rysuj()
        {
            Console.Title = "Gra Życie";

            Console.ResetColor();
            Console.SetCursorPosition(0, 0);
            Console.WriteLine("Liczba żywych      : {0} / {1} [{2:F1}%]             ", liczbaZywych,szerokosc*Wysokosc,100.0*(double)liczbaZywych/(double)(szerokosc*Wysokosc));
            Console.WriteLine("Cykl               : {0}                             ", numerCyklu);
            Console.WriteLine("Wymiary            : {0}x{1}                         ", Szerokosc,Wysokosc);
    
            int startx = 3;
            int starty = 5;

            Console.ForegroundColor = kolorZnakow;
            Console.BackgroundColor = kolorTla;
            
            for (int i = 0; i < szerokosc; i++)
                for (int j = 0; j < wysokosc; j++)
                {
                    Console.SetCursorPosition(startx + i, starty + j );
                    if (plansza[i, j] == 1) Console.Write(znak);
                    else Console.Write(' ');
                }

            Console.ResetColor();
            Console.WriteLine("\n");
        }

        private int LiczbaSasiadow(int x, int y)
        {
            int ls = 0;
            ls += plansza[(x + szerokosc - 1) % szerokosc, (y + Wysokosc - 1) % Wysokosc]%2;
            ls += plansza[(x + szerokosc - 1) % szerokosc, (y + Wysokosc) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc - 1) % szerokosc, (y + Wysokosc + 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc) % szerokosc, (y + Wysokosc - 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc) % szerokosc, (y + Wysokosc + 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc + 1) % szerokosc, (y + Wysokosc - 1) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc + 1) % szerokosc, (y + Wysokosc) % Wysokosc] % 2;
            ls += plansza[(x + szerokosc + 1) % szerokosc, (y + Wysokosc + 1) % Wysokosc] % 2;

            return ls;
        }

        public void Cykl()
        {
            int n = liczbaZywych;
            for (int i = 0; i < szerokosc; i++)
            {
                for (int j = 0; j < wysokosc; j++)
                {
                    int x = LiczbaSasiadow(i, j);
                    if (plansza[i, j] == 0 && x == 3)
                    {
                        plansza[i, j] = 3;            // narodziny  
                        n++;
                    }
                    if (plansza[i, j] == 1 && (x < 2 || x > 3))
                    {
                        plansza[i, j] = 2;    // śmierć
                        n--;
                    }
                }
            }

            for (int i = 0; i < szerokosc; i++)
                for (int j = 0; j < wysokosc; j++)
                    plansza[i, j] %= 2;

            numerCyklu++;
            liczbaZywych = n;
        }

        public bool Zapisz(string nazwapliku)
        {
            using (StreamWriter plik = new StreamWriter(nazwapliku))
            {
                plik.WriteLine("Liczba żywych      : {0} / {1} [{2:F1}%]             ", liczbaZywych, szerokosc * Wysokosc, 100.0 * (double)liczbaZywych / (double)(szerokosc * Wysokosc));
                plik.WriteLine("Cykl               : {0}                             ", numerCyklu);
                plik.WriteLine("Wymiary            : {0}x{1}                         ", Szerokosc, Wysokosc);
                
                for (int i = 0; i < wysokosc; i++)
                {
                    for (int j = 0; j < szerokosc; j++)
                    {
                        if (plansza[i, j] == 1) plik.Write(znak);
                        else plik.Write(' ');
                    }
                    plik.WriteLine();
                }
            }
            return true;
        }
        
    }
}
